package gui;

import java.awt.Color;
import java.awt.Graphics;
import javax.swing.JPanel;
import pecas.Peca;

/**
 * 2014-06-10
 *
 * @author Emerson Ribeiro de Mello
 */
public class Tabuleiro extends JPanel {

    private final JogoDeXadrez pai;
    private Peca[][] tabuleiro; // matriz com todas as peças
    private int selColuna, selLinha; // coordenadas da peça selecionada
    private boolean jogadorDaVez; // indica qual é o jogador da vez

    public Tabuleiro(JogoDeXadrez p) {
        this.pai = p;
        // alterar a cor de fundo do painel
        this.setBackground(Color.WHITE);

        this.tabuleiro = new Peca[8][8]; // lin x col

        // Distribuindo as peças pelo tabuleiro
        for (int lin = 0; lin < 8; lin++) { //percorrendo linhas
            // determinando a cor da peca
            // false = preta
            // true = branca
            boolean cor = (lin > 4);

            for (int col = 0; col < 8; col++) { //percorrendo colunas
                this.tabuleiro[lin][col] = new Peca(cor, col, lin);
            }
            // se já preencheu as 2 primeiras linhas, pule para preencher as 
            // 2 últimas linhas
            if (lin == 1) {
                lin = 5;
            }
        }

        this.jogadorDaVez = true;

        selColuna = -1;
        selLinha = -1;
        this.repaint();
    }

    @Override
    public void paint(Graphics g) {
        super.paint(g);
        this.desenharTabuleiro(g);
        this.preencherPecas(g);
        g.dispose();
    }

    /**
     * Este método desenha o tabuleiro sem as peças
     *
     * @param g
     */
    public void desenharTabuleiro(Graphics g) {
        boolean casaPreenchida = false;
        g.setColor(Color.gray);

        for (int i = 0; i < 400; i += 50) {
            for (int j = 0; j < 400; j += 50) {
                if (casaPreenchida == false) {
                    g.setColor(Color.white);
                    g.fillRect(i, j, 50, 50);
                } else {
                    g.setColor(Color.gray);
                    g.fillRect(i, j, 50, 50);
                }
                casaPreenchida = !casaPreenchida;
            }
            casaPreenchida = !casaPreenchida;
        }
    }

    /**
     * Método responsável por selecionar peça e fazer a movimentação. Se na casa
     * onde ocorreu o clique houver uma peça do jogador, então deve-se desenhar
     * um retângulo laranja como destaque na casa e registrar o início do
     * movimento do jogador. Se este método for invocado após a seleção da peça,
     * então deve-se verificar se trata-se de uma simples movimentação de peças,
     * ou se houver uma captura de peças do adversário
     *
     * @param col coluna
     * @param lin linha
     */
    public void mover(int col, int lin) {

        if (tabuleiro[lin][col] != null) { // se clicou em uma casa com peça

            // Se clicou em uma casa que contém a peça do jogador da vez,
            // então selecione a peça e guarde suas coordenadas x e y
            if (tabuleiro[lin][col].isCor() == this.jogadorDaVez) {
                Graphics g = this.getGraphics();

                // se houver peça do jogador selecionada, apague seleção anterior
                if (selColuna != -1) {
                    Color c = Color.gray;
                    if (((selColuna % 2 == 0) && selLinha % 2 == 0)
                            || ((selColuna % 2 != 0) && selLinha % 2 != 0)) {
                        c = Color.white;
                    }

                    g.setColor(c);
                    g.drawRoundRect(selColuna * 50 + 5, selLinha * 50 + 5, 40, 40, 5, 5);
                }
                // fazendo a seleção
                g.setColor(Color.orange);
                g.drawRoundRect(col * 50 + 5, lin * 50 + 5, 40, 40, 5, 5);
                selColuna = col;
                selLinha = lin;
            }
        } else { // Se clicou em uma casa vazia
            // se houver peça do jogador selecionada
            // entao, indica que é uma movimentação de peça
            if (selColuna != -1) {

                // Faça a peça executar sua lógica de movimentação
                if (tabuleiro[selLinha][selColuna].mover(lin, col, tabuleiro)) {
                    this.selColuna = -1;
                    // dá a vez para o outro jogador
                    this.jogadorDaVez = !this.jogadorDaVez;
                    this.repaint();
                }

            }
        }

    }

    /**
     * Pede para cada peça se desenhar em suas coordenadas
     *
     * @param g
     */
    public void preencherPecas(Graphics g) {

        for (int lin = 0; lin < 8; lin++) {
            for (int col = 0; col < 8; col++) {
                if (this.tabuleiro[lin][col] != null) {
                    this.tabuleiro[lin][col].desenhar(g, this);
                }
            }
        }
    }

}
